@@ -1,9 +1,8 @@
 package Test::Smoke::Reporter;
 use strict;
 
-# $Id: Reporter.pm 1363 2012-05-10 21:21:26Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.050';
+$VERSION = '0.052';
 
 require File::Path;
 require Test::Smoke;
@@ -12,11 +11,14 @@ use File::Spec::Functions;
 use JSON;
 use LWP::UserAgent;
 use POSIX qw( strftime );
+use Encode qw( decode encode );
 use Test::Smoke::SysInfo;
 use Text::ParseWords;
-use Test::Smoke::Util qw( grepccmsg get_smoked_Config
+use Test::Smoke::Util qw( grepccmsg get_smoked_Config read_logfile
                           time_in_hhmm get_local_patches );
 
+use constant USERNOTE_ON_TOP => 'top';
+
 my %CONFIG = (
     df_ddir         => curdir(),
     df_outfile      => 'mktest.out',
@@ -36,6 +38,8 @@ my %CONFIG = (
 
     df_v            => 0,
     df_user_note    => '',
+    df_un_file      => undef,
+    df_un_position  => 'bottom', # != USERNOTE_ON_TOP for bottom
 );
 
 =head1 NAME
@@ -248,6 +252,9 @@ sub _parse {
             $rpt{patchdescr} = $descr || $pl;
             next;
         }
+        if (/^Smoking branch (\S+)/) {
+            $rpt{smokebranch} = $1;
+        }
 
         if (/^MANIFEST /) {
             push @{$self->{_mani}}, $_;
@@ -293,7 +300,11 @@ sub _parse {
             next;
         }
 
-        if (m/(?:PERLIO|TSTENV)\s*=\s*([-\w:.]+)/) {
+        if (m/(?:PERLIO|TSTENV)\s*=\s*([-\w:.]+)/
+              # skip this if it's from a build failure, since the
+              # Unable to build... pushed an M
+              && (!@{$new[-1]{results}}
+                  || $new[-1]{results}[0]{summary} ne "M")) {
             $tstenv          = $1;
             $previous_failed = "";
             $rpt{$cfgarg}->{summary}{$debug}{$tstenv} ||= "?";
@@ -352,20 +363,32 @@ sub _parse {
                 $new[-1]{results}[-1]{summary} = "X";
             }
             push @{$rpt{$cfgarg}->{$debug}{$tstenv}{failed}}, $_;
-            push(
-                @{$new[-1]{results}[-1]{failures}},
-                m/^ \s* (\S+?) \.+ (\w+) \s* $/x
-                    ? {
-                        test   => $1,
-                        status => $2,
-                        extra  => []
-                        }
-                    : {
-                        test   => "?",
-                        status => "?",
-                        extra  => []
-                    }
-            );
+            while (m/^ \s* (\S+?) \s* \.+(?:\s+\.+)* \s* (\w.*?) \s*$/xgm) {
+                my ($_test, $_info) = ($1, $2);
+
+                push(
+                    @{$new[-1]{results}[-1]{failures}},
+                    $_info =~ m/^ \w+ $/x
+                        ? {
+                            test   => $_test,
+                            status => $_info,
+                            extra  => []
+                            }
+                        : # TEST output from minitest
+                    $_info =~ m/^ (\w+) \s+at\ test\s+ (\d+) \s* $/x
+                 || $_info =~ m/^ (\w+)--(\S.*\S) \s* $/x 
+                        ? {
+                            test   => $_test,
+                            status => $1,
+                            extra  => [ $2 ]
+                            }
+                        : {
+                            test   => "?",
+                            status => "?",
+                            extra  => []
+                            }
+                );
+            }
         }
 
         if (/^Finished smoking [\dA-Fa-f]+/) {
@@ -406,7 +429,7 @@ sub _parse {
 
             if ($previous_failed ne $_) {
                 if (not $rpt{$cfgarg}->{summary}{$debug}{$tstenv}
-                    or $rpt{$cfgarg}->{summary}{$debug}{$tstenv} ne "X")
+                    or $rpt{$cfgarg}->{summary}{$debug}{$tstenv} !~ m/[XM]/)
                 {
                     $rpt{$cfgarg}->{summary}{$debug}{$tstenv} = "F";
                     $new[-1]{results}[-1]{summary} = "F";
@@ -414,7 +437,12 @@ sub _parse {
                 push @{$rpt{$cfgarg}->{$debug}{$tstenv}{failed}}, $_;
                 push(
                     @{$new[-1]{results}[-1]{failures}},
-                    m/^ \s* (\S+?) \.+ (\w+) \s* $/x
+                    m{^ \s*                     # leading space
+                       ((?:\S+[/\\])?           # Optional leading path to
+                           \S(?:[^.]+|\.t)+)    #  test file name
+                       [. ]+                    # ....... ......
+                       (\w.*?)                  # result
+                       \s* $}x
                         ? {
                             test   => $1,
                             status => $2,
@@ -442,7 +470,7 @@ sub _parse {
             push @{$rpt{$cfgarg}->{$debug}{$tstenv}{passed}}, $_;
             push(
                 @{$new[-1]{results}[-1]{failures}},
-                m/^ \s* (\S+?) \.+ (\w+) \s* $/x
+                m/^ \s* (\S+?) \.+(?:\s+\.+)* (\w+) \s* $/x
                     ? {
                         test   => $1,
                         status => $2,
@@ -495,9 +523,16 @@ sub _post_process {
     my $self = shift;
 
     unless (defined $self->{is56x}) {
-        my %cfg = get_smoked_Config($self->{ddir}, "version");
-        my $p_version = sprintf "%d.%03d%03d", split m/\./, $cfg{version};
-        $self->{is56x} = $p_version < 5.007;
+        $self->{is56x} = 0;
+        # Overly defensive, as .out files might be analyzed outside of the
+        # original smoke environment
+        if ($self->{ddir} && -d $self->{ddir}) {
+            my %cfg = get_smoked_Config($self->{ddir}, "version");
+            if ($cfg{version} =~ m/^\s* ([0-9]+) \. ([0-9]+) \. ([0-9]+) \s*$/x) {
+                my $p_version = sprintf "%d.%03d%03d", $1, $2, $3;
+                $self->{is56x} = $p_version < 5.007;
+            }
+        }
     }
     $self->{defaultenv} ||= $self->{is56x};
 
@@ -735,6 +770,8 @@ locally in the file mktest.jsn
 
 sub smokedb_data {
     my $self = shift;
+    $self->{v} and print "Gathering CoreSmokeDB information...\n";
+
     my %rpt  = map { $_ => $self->{$_} } keys %$self;
     $rpt{manifest_msgs}   = delete $rpt{_mani};
     $rpt{applied_patches} = [$self->registered_patches];
@@ -752,6 +789,7 @@ sub smokedb_data {
             duration         => $self->{_rpt}{secs},
             git_describe     => $self->{_rpt}{patchdescr},
             git_id           => $self->{_rpt}{patch},
+            smoke_branch     => $self->{_rpt}{smokebranch},
             hostname         => $si->host,
             lang             => $ENV{LANG},
             lc_all           => $ENV{LC_ALL},
@@ -761,7 +799,7 @@ sub smokedb_data {
             reporter         => $self->{_conf_args}{from},
             reporter_version => $VERSION,
             smoke_date       => __posixdate($self->{_rpt}{started}),
-            smoke_revision   => $Test::Smoke::REVISION,
+            smoke_revision   => $Test::Smoke::VERSION,
             smoker_version   => $Test::Smoke::Smoker::VERSION,
             smoke_version    => $Test::Smoke::VERSION,
             test_jobs        => $ENV{TEST_JOBS},
@@ -781,12 +819,8 @@ sub smokedb_data {
         if (   ($send_log eq "always")
             or ($send_log eq "on_fail" && $rpt_fail))
         {
-            local *FH;
-            if (open FH, "<", $rpt{lfile}) {
-                local $/;
-                $rpt{log_file} = <FH>;
-                close FH;
-            }
+            my $log = read_logfile($self, $rpt{lfile});
+            $log and $rpt{log_file} = $log;
         }
     }
     $rpt{out_file} = undef;
@@ -807,11 +841,15 @@ sub smokedb_data {
     my $json = JSON->new->utf8(1)->pretty(1)->encode(\%rpt);
 
     # write the json to file:
-    local *JSN;
-    if (open JSN, ">", catfile($self->{ddir}, $self->{jsnfile})) {
-        binmode(JSN);
-        print JSN $json;
-        close JSN;
+    my $jsn_file = catfile($self->{ddir}, $self->{jsnfile});
+    if (open my $jsn, ">", $jsn_file) {
+        binmode($jsn);
+        print {$jsn} $json;
+        close $jsn;
+        $self->{v} and print "Write to '$jsn_file': ok\n";
+    }
+    else {
+        print "Error creating '$jsn_file': $!\n";
     }
 
     return $self->{_json} = $json;
@@ -826,6 +864,7 @@ Return a string with the full report
 sub report {
     my $self = shift;
     return unless defined $self->{_outfile};
+    $self->_get_usernote();
 
     my $report = $self->preamble;
 
@@ -858,6 +897,29 @@ sub report {
     return $report;
 }
 
+=item $reporter->_get_usernote()
+
+Return $self->{user_note} if exists.
+
+Check if C<< $self->{un_file} >> exists, and read contents into C<<
+$self->{user_note} >>.
+
+=cut
+
+sub _get_usernote {
+    my $self = shift;
+
+    if (!$self->{user_note} && $self->{un_file}) {
+        if (open my $unf, '<', $self->{un_file}) {
+            $self->{user_note} = join('', <$unf>);
+        }
+        else {
+            print "Cannot read '$self->{un_file}': $!\n" if $self->{v};
+        }
+    }
+    $self->{user_note} =~ s/(?<=\S)\s*\z/\n/;
+}
+
 =item $reporter->ccinfo( )
 
 Return the string containing the C-compiler info.
@@ -946,21 +1008,28 @@ Use a port of Jarkko's F<grepccerr> script to report the compiler messages.
 
 sub ccmessages {
     my $self = shift;
+
     my $ccinfo = $self->{_rpt}{cinfo} || $self->{_ccinfo} || "cc";
     $ccinfo =~ s/^(.+)\s+version\s+.+/$1/;
 
     $^O =~ /^(?:linux|.*bsd.*|darwin)/ and $ccinfo = 'gcc';
     my $cc = $ccinfo =~ /(gcc|bcc32)/ ? $1 : $^O;
 
-    $self->{v} and print "Looking for cc messages: '$cc'\n";
-    my $errors = grepccmsg($cc, $self->{lfile}, $self->{v}) || [];
+    if (!$self->{_ccmessages_}) {
+
+        $self->{v} and print "Looking for cc messages: '$cc'\n";
+        $self->{_ccmessages_} = grepccmsg($cc, $self->{lfile}, $self->{v}) || [];
+    }
+
+    return @{$self->{_ccmessages_}} if wantarray;
+    return "" if !$self->{_ccmessages_};
 
     local $" = "\n";
-    return @$errors ? wantarray ? @$errors : <<EOERRORS : "";
+    return <<"    EOERRORS";
 
 Compiler messages($cc):
-@$errors
-EOERRORS
+@{$self->{_ccmessages_}}
+    EOERRORS
 }
 
 =item $reporter->preamble( )
@@ -991,14 +1060,26 @@ sub preamble {
 
     my $os = $si->os;
 
-    return <<__EOH__;
-Automated smoke report for $Config{version} patch $self->{_rpt}{patchlevel}
+    my $branch = '';
+    if ($self->{_rpt}{smokebranch}) {
+        $branch = " branch $self->{_rpt}{smokebranch}";
+    }
+
+    my $preamble = <<__EOH__;
+Automated smoke report for$branch $Config{version} patch $self->{_rpt}{patchlevel}
 $this_host: $cpu ($archname)
     on        $os
     using     $cinfo
     smoketime $time_msg (average $savg_msg)
 
 __EOH__
+
+    if ($self->{un_position} eq USERNOTE_ON_TOP) {
+        (my $user_note = $self->{user_note}) =~ s/(?<=\S)\s*\z/\n/;
+        $preamble = "$user_note\n$preamble";
+    }
+
+    return $preamble;
 }
 
 =item $reporter->smoke_matrix( )
@@ -1200,14 +1281,20 @@ __EOL__
 sub signature {
     my $self = shift;
     my $this_pver = $^V ? sprintf "%vd", $^V : $];
-    my $build_info = "$Test::Smoke::VERSION build $Test::Smoke::REVISION";
-    (my $user_note = $self->{user_note} || "") =~ s/(\S)[\s\r\n]*\z/$1\n/;
-    return <<__EOS__
-$user_note
+    my $build_info = "$Test::Smoke::VERSION";
+
+    my $signature = <<"    __EOS__";
 -- 
 Report by Test::Smoke v$build_info running on perl $this_pver
 (Reporter v$VERSION / Smoker v$Test::Smoke::Smoker::VERSION)
-__EOS__
+    __EOS__
+
+    if ($self->{un_position} ne USERNOTE_ON_TOP) {
+        (my $user_note = $self->{user_note}) =~ s/(?<=\S)\s*\z/\n/;
+        $signature = "\n$user_note\n$signature";
+    }
+
+    return $signature;
 }
 
 1;
